import yaml

from si_tests import logger
from si_tests.deployments.utils.namespace import NAMESPACE
from si_tests.utils import waiters, utils

LOG = logger.logger


def get_compute_metadata(ostcm, node_name):
    metadata = {}
    for resource in ["compute_service", "network_agent", "volume_service"]:
        metadata[resource] = getattr(ostcm, resource).list(["--host", node_name])

    return metadata


def get_controller_metadata(ostcm, node_name):
    metadata = {}
    for resource in ["network_agent", "volume_service"]:
        metadata[resource] = getattr(ostcm, resource).list(["--host", node_name])

    return metadata


def _test_node_deletion_request(ostcm, get_metadata, **kwargs):
    k8scluster = ostcm.os_manager.api
    node_name = kwargs.get("node_name", k8scluster.nodes.list(label_selector=kwargs.get("node_label"))[0].name)

    assert any(get_metadata(ostcm, node_name).values()), f"There is no metadata on the {node_name} node"

    LOG.info("Creating node deletion request")
    body = {
        "apiVersion": "lcm.mirantis.com/v1alpha1",
        "kind": "NodeDeletionRequest",
        "metadata": {"name": node_name},
        "spec": {"nodeName": node_name},
    }
    ndr = k8scluster.nodedeletionrequests.create(body=body)
    LOG.info("Waiting for inactive node workload lock")
    waiters.wait(
        lambda: k8scluster.nodeworkloadlocks.get(f"openstack-{node_name}").state == "inactive",
        timeout=120,
        interval=10,
    )

    node = k8scluster.nodes.get(node_name)

    LOG.info("Blocking up the node (mark it as unschedulable)")
    node.cordon()

    LOG.info("Draining the node (move out pods except DaemonSets)")
    pods = [
        pod
        for pod in k8scluster.pods.list(namespace=NAMESPACE.openstack, field_selector=f"spec.nodeName={node_name}")
        if not pod.get_owner_reference().startswith("DaemonSet/")
    ]
    if pods:
        for pod in pods:
            LOG.info(f"Moving out the {pod.namespace}/{pod.name}")
            pod.eviction()

        LOG.info("Waiting for moved pods out")

        def _status_msg_pods():
            remained_pods = [f"{pod.namespace}/{pod.name}" for pod in pods if pod.exists()]
            return f"\n{yaml.dump(remained_pods)}"

        waiters.wait(
            lambda: all([not pod.exists() for pod in pods]),
            timeout=900,
            interval=30,
            status_msg_function=_status_msg_pods,
        )

    LOG.info("Deleting the node and node deletion request")
    node.delete()
    ndr.delete()

    LOG.info("Waiting for node workload lock to be removed")
    waiters.wait(
        lambda: not [
            nwl.uid for nwl in k8scluster.nodeworkloadlocks.list_all() if nwl.name == f"openstack-{node_name}"
        ],
        timeout=180,
        interval=20,
    )

    LOG.info("Checking out all the metadata is cleaned up")
    assert not any(get_metadata(ostcm, node_name).values()), f"\n{yaml.dump(get_metadata(ostcm, node_name))}"
    LOG.info("All the metadata has been cleaned up")


def test_node_deletion_request_compute(openstack_client_manager):
    """Test node deletion process of compute node

    - check out original OpenStack metadata
    - create NodeDeletionRequest
    - wait for inactive NodeWorkloadLock
    - cordon/drain Node
    - wait for Pods are evicted
    - delete Node
    - delete NodeDeletionRequest
    - check out OpenStack metadata is cleaned up

    Parameters required:
      - KUBECONFIG
    """
    _test_node_deletion_request(
        openstack_client_manager,
        get_compute_metadata,
        node_name=sorted(
            openstack_client_manager.os_manager.api.nodes.list(label_selector=utils.NodeLabel.os_compute.value),
            key=lambda n: n.data["metadata"]["creation_timestamp"],
            reverse=True,
        )[0].name,
    )


def test_node_deletion_request_controller(openstack_client_manager):
    """Test node deletion process of controller node

    - check out original OpenStack metadata
    - create NodeDeletionRequest
    - wait for inactive NodeWorkloadLock
    - cordon/drain Node
    - wait for Pods are evicted
    - delete Node
    - delete NodeDeletionRequest
    - check out OpenStack metadata is cleaned up

    Parameters required:
      - KUBECONFIG
    """
    MARIADB_POD_NAME = "mariadb-server-2"

    _test_node_deletion_request(
        openstack_client_manager,
        get_controller_metadata,
        node_name=openstack_client_manager.os_manager.api.pods.get(MARIADB_POD_NAME, NAMESPACE.openstack).node_name,
    )
